//
//  Heap.m
//  Heap
//
//  Created by Grassi, Dan on 8/20/12.
//  Copyright (c) 2012 Zaph, llc. All rights reserved.
//

#import "Heap.h"

@implementation HeapItem
static int itemID = 0;
- (id)init {
    self = [super init];
    if (self) {
        _itemID =  ++itemID;
    }
    return self;
}

+ (id)heapItemWithValue:(int)value {
    HeapItem *item = [HeapItem new];
    item.value = value;
    return item;
}

- (NSString *)description {
    NSMutableString *text = [NSMutableString new];
    [text appendFormat:@"HeapItem: "];
    [text appendFormat:@"itemID:%3i, ", self.itemID];
    [text appendFormat:@"value: %3i, ", self.value];
    [text appendFormat:@"text:  %@", self.text];
    return text;
}

@end


@interface Heap ()
@property (strong, nonatomic) NSMutableArray *array;
@property (strong, nonatomic) NSArray        *previousArray;
@end

@implementation Heap

- (id)init {
    self = [super init];
    if (self) {
        _array = [NSMutableArray new];
        _previousArray = [NSArray new];
        _delay = NO;
    }
    return self;
}

- (NSUInteger)count {
    return self.array.count;
}

- (int)nextItemID {
    return itemID+1;
}

- (HeapItem *)insertItemWithValue:(int)value {
    HeapItem *item = [HeapItem heapItemWithValue:value];
    [self.array addObject:item];

    HeapItem *parent = [self parentForChild:item];
    [self bubbleUpItem:item parent:parent];
    [self swapDelay];

    return item;
}

- (void)bubbleUpItem:(HeapItem *)item parent:(HeapItem *)parent {
    while (parent && item.value < parent.value) {
        [self swap:item with:parent];
        parent = [self parentForChild:item];
    }
}

- (HeapItem *)peekItem {
    return [self topObject];
}

- (HeapItem *)deleteItem {
    HeapItem *topItem  = [self topObject];
    HeapItem *item     = [self lastObject];
    [self swap:topItem with:item];
    [self.array removeLastObject];

    HeapItem *leftItem = [self leftChildForParent:item];
    [self bubbleDownItem:item leftItem:leftItem];
    [self swapDelay];

    return topItem;
}

- (void)bubbleDownItem:(HeapItem *)item leftItem:(HeapItem *)leftItem {
    while (leftItem) {
        HeapItem *rightItem = [self rightChildForParent:item];
        HeapItem *swapee = [self swapee:leftItem with:rightItem];
        if (item.value <= swapee.value)
            break;

        [self swap:item with:swapee];
        leftItem = [self leftChildForParent:item];
    }
}

- (HeapItem *)swapee:(HeapItem *)leftItem with:(HeapItem *)rightItem {
    if (rightItem == nil ||
        leftItem.value < rightItem.value)
        return leftItem;
    else
        return rightItem;
}

- (void)swap:(HeapItem *)item1 with:(HeapItem *)item2 {
    [self swapDelay];

    NSUInteger index1 = [self indexForObject:item1];
    NSUInteger index2 = [self indexForObject:item2];
    if (index1 == NSNotFound || index2 == NSNotFound)
        return;
    [self.array exchangeObjectAtIndex:index1-1 withObjectAtIndex:index2-1];
}

- (void)swapDelay {
    if (self.delay) {
        [[NSNotificationCenter defaultCenter] postNotificationName:@"updateTreeView" object:self];
        NSDate* dateLimit = [NSDate dateWithTimeIntervalSinceNow:self.delay];
        [[NSRunLoop currentRunLoop] runUntilDate:dateLimit];
    }
}

- (id)parentForChild:(HeapItem *)child {
    NSUInteger childIndex = [self indexForObject:child];
    NSUInteger parentIndex = [self parentIndexForChildIndex:childIndex];
    return [self objectForIndex:parentIndex];
}

- (id)leftChildForParent:(HeapItem *)parent {
    NSUInteger parentIndex = [self indexForObject:parent];
    NSUInteger childIndex = [self leftChildIndexForParentIndex:parentIndex];
    return [self objectForIndex:childIndex];
}

- (HeapItem *)rightChildForParent:(HeapItem *)parent {
    NSUInteger parentIndex = [self indexForObject:parent];
    NSUInteger childIndex = [self rightChildIndexForParentIndex:parentIndex];
    return [self objectForIndex:childIndex];
}

- (HeapItem *)topObject {
    return [self objectForIndex:1];
}

- (HeapItem *)lastObject {
    return [self.array lastObject];
}

- (NSUInteger)parentIndexForChildIndex:(NSUInteger)childIndex {
    return childIndex/2;
}

- (NSUInteger)leftChildIndexForParentIndex:(NSUInteger)parentIndex {
    return parentIndex*2;
}

- (NSUInteger)rightChildIndexForParentIndex:(NSUInteger)parentIndex {
    return parentIndex*2 + 1;
}


- (NSUInteger)indexForObject:(HeapItem *)object {
    NSUInteger index = [self.array indexOfObject:object];
    if (index == NSNotFound)
        return index;
    else
        return index+1;
}

- (HeapItem *)objectForIndex:(NSUInteger)index {
    if (index <= self.array.count && index >= 1)
        return self.array[index-1];
    else
        return nil;
}

- (NSString *)description {
    NSMutableString *text = [NSMutableString new];
    [text appendFormat:@"Heap: "];
    for (HeapItem *item in self.array) {
        [text appendFormat:@"%i ", item.itemID];
    }
    return text;
}

- (NSString *)descriptionByValue {
    NSMutableString *text = [NSMutableString new];
    [text appendFormat:@""];
    for (HeapItem *item in self.array) {
        [text appendFormat:@"%2i ", item.value];
    }
    return text;
}

- (NSAttributedString *)attributedDescriptionByValue {
    NSDictionary *redAttribute   = @{NSForegroundColorAttributeName : [UIColor redColor]};
    
    NSMutableAttributedString *attributedString = [NSMutableAttributedString new];
    for (int i=0; i<self.array.count; i++) {
        int item = ((HeapItem *)self.array[i]).value;
        int previousItem = 0;
        if (i < self.previousArray.count)
            previousItem = ((HeapItem *)self.previousArray[i]).value;

        NSMutableAttributedString *itemAS = [self attributedStringWithValue:item];
        NSRange range = NSMakeRange(0, itemAS.length-1);
        if (item != previousItem) {
            [itemAS addAttributes:redAttribute range:range];
        }

        [attributedString appendAttributedString:itemAS];
    }
    self.previousArray = [self.array copy];

    return attributedString;
}

- (NSMutableAttributedString *)attributedStringWithValue:(int)value {
    NSDictionary *fontAttribute  = @{NSFontAttributeName : [UIFont fontWithName:@"Courier" size:15.0]};
    NSString *itemString = [NSString stringWithFormat:@"%i ", value];
    NSMutableAttributedString *itemAS = [[NSMutableAttributedString alloc] initWithString:itemString attributes:fontAttribute];
    return itemAS;
}

- (NSUInteger)countByEnumeratingWithState:(NSFastEnumerationState *)state objects:(__unsafe_unretained id *)stackbuf count:(NSUInteger)len {
    return [self.array countByEnumeratingWithState:state objects:stackbuf count:len];
}

@end
