//
//  TreeView.m
//  Tree
//
//  Created by Grassi, Dan on 8/14/12.
//  Copyright (c) 2012 Zaph, llc. All rights reserved.
//

#import "TreeView.h"
#import "Node.h"

@interface TreeView ()
@property (assign, nonatomic) CGContextRef context;
@property (assign, nonatomic) CGFloat startX;
@property (assign, nonatomic) CGFloat stepX;
@property (assign, nonatomic) CGFloat startY;
@property (assign, nonatomic) CGFloat stepY;
@property (assign, nonatomic) CGFloat insetX;
@property (assign, nonatomic) CGRect  graphRect;
@property (strong, nonatomic) UIColor *drawColor;
@property (strong, nonatomic) UIColor *highlightColor;
@end


@implementation TreeView

- (void)awakeFromNib {
    self.vertexSize = 10;
    self.inset = self.vertexSize*2;
    self.lineWidth = 1;
    self.fontSize = 10;
    self.drawColor = [UIColor blackColor];
    self.highlightColor = [UIColor redColor];
}

- (void)drawRect:(CGRect)rect {
    self.context = UIGraphicsGetCurrentContext();
    CGContextSetLineWidth(self.context, self.lineWidth);
    
    [self drawGraphOfNode:self.treeDataModel.topNode];
}

- (void)drawGraphOfNode:(Node *)node {
    [self initializeGraphState:node];
    [self connectNodesFromParent:self.treeDataModel.topNode];
}

- (void)initializeGraphState:(Node *)node {
    self.graphRect = CGRectInset(self.bounds, self.vertexSize*2.0, self.vertexSize*2.0);
    self.startX = self.graphRect.size.width / 2.0 + self.vertexSize * 2.0;
    if (node.width == 1) {
        self.stepX = self.graphRect.size.width / 2.0;
        self.insetX = self.inset + self.graphRect.size.width / 2.0;
    }
    else {
        self.stepX  = self.graphRect.size.width / (node.width-1.0);
        self.insetX = self.inset;
    }
    self.startY = self.graphRect.origin.y;
    self.stepY  = self.graphRect.size.height / MAX((self.treeDataModel.depth), 1.0);
}

- (void)connectNodesFromParent:(Node *)node {
    CGFloat startLocation = (node.width-1)/2.0;
    [self connectNodesFromParent:node parentLocation:startLocation firstLocation:0];
    
    if (node == nil)
        return;
    
    CGPoint nodePoint = [self pointForLocation:startLocation level:node.level];
    [self drawNodeAtPoint:nodePoint node:node];
}

- (void)connectNodesFromParent:(Node *)parent parentLocation:(CGFloat)parentLocation firstLocation:(CGFloat)firstLocation {
    int childLocationOffsetX = 0;
    for (Node *child in parent) {
        CGFloat nextLocation = firstLocation + childLocationOffsetX;
        CGFloat childLocation = nextLocation + (child.width-1)/2.0;
        
        CGPoint parentPoint = [self pointForLocation:parentLocation level:parent.level];
        CGPoint childPoint  = [self pointForLocation:childLocation  level:child.level];
        [self drawLineFrom:parentPoint to:childPoint];
        [self connectNodesFromParent:child parentLocation:childLocation firstLocation:nextLocation];
        [self drawNodeAtPoint:childPoint node:child];
        
        childLocationOffsetX += child.width;
    }
}

- (CGPoint)pointForLocation:(CGFloat)location level:(int)level {
    return CGPointMake(self.insetX + location * self.stepX, self.startY + level * self.stepY);
}

- (void)drawNodeAtPoint:(CGPoint)point node:(Node *)node {
    if (node.text.length) {
        [self drawBadgeWithNode:node point:point];
    }
    else {
        CGRect vertexRect = [self vertexRectForPoint:point];
        CGContextFillEllipseInRect(self.context, vertexRect);
    }
}

- (void)drawBadgeWithNode:(Node *)node point:(CGPoint)point {
    UIFont *font = [UIFont systemFontOfSize:self.fontSize];
    int widthExtra = self.fontSize/4.0;
    
    CGSize textSize = [node.text sizeWithFont:font];
    CGRect rrFrame = CGRectMake(point.x, point.y, textSize.width, textSize.height);
    rrFrame = CGRectInset(rrFrame, -self.lineWidth-widthExtra, -self.lineWidth);
    rrFrame.origin = CGPointMake(point.x-(rrFrame.size.width/2.0), point.y-(rrFrame.size.height/2.0));
    
    BOOL highlight = self.highlightID == node.value;
    [self drawEmptyBadgeRect:rrFrame highlight:highlight];
    
    rrFrame.origin.x += self.lineWidth + widthExtra;
    rrFrame.origin.y += self.lineWidth;
    [node.text drawInRect:rrFrame withFont:font];
}

- (void)drawEmptyBadgeRect:(CGRect)rect highlight:(BOOL)highlight {
    CGContextSaveGState(self.context);

    CGFloat lineWidth = [self lineWidthColorForHighlight:highlight];
    [self drawOutlineRect:rect WithLineWidth:lineWidth];

    CGContextRestoreGState(self.context);
}

- (void)drawOutlineRect:(CGRect)rect WithLineWidth:(CGFloat)lineWidth {
    CGFloat cornerRadius = rect.size.height/2.0;
    UIBezierPath *roundedRectPath = [UIBezierPath bezierPathWithRoundedRect:rect cornerRadius:cornerRadius];
    [roundedRectPath fill];
    roundedRectPath.lineWidth = lineWidth;
    [roundedRectPath stroke];
}

-(CGFloat)lineWidthColorForHighlight:(BOOL)highlight {
    [self.backgroundColor setFill];
    UIColor *color    = (highlight) ? self.highlightColor :self.drawColor;
    [color setStroke];
    
    return (highlight) ? self.lineWidth*2.0 : self.lineWidth;
}

- (void)drawLineFrom:(CGPoint)from to:(CGPoint)to {
    CGContextMoveToPoint(self.context, from.x, from.y);
    CGContextAddLineToPoint(self.context, to.x, to.y);
    CGContextStrokePath(self.context);
}

-(CGRect)vertexRectForPoint:(CGPoint)point {
    return CGRectMake(point.x-self.vertexSize/2.0, point.y-self.vertexSize/2.0, self.vertexSize, self.vertexSize);
}

-(CGRect)vertexRectForX:(CGFloat)x y:(CGFloat)y {
    return CGRectMake(x-self.vertexSize/2.0, y-self.vertexSize/2.0, self.vertexSize, self.vertexSize);
}

@end
